import os
from typing import List
from datetime import datetime
from sqlalchemy import text
from sqlalchemy.exc import SQLAlchemyError
import logging
import requests
from mcp.server import FastMCP

logger = logging.getLogger(__name__)

mcp = FastMCP()


def fetch_readme_content(github_url: str) -> str:
    """
    获取GitHub仓库的README内容
    Args:
        github_url: GitHub仓库URL
    Returns:
        README文件的内容
    """
    # 移除末尾的斜杠(如果有)
    clean_url = github_url.rstrip('/')
    logger.info(f"Fetching README from {clean_url}")

    def build_readme_url(branch: str) -> str:
        """构建不同分支的README URL"""
        if f'/tree/{branch}/src/' in clean_url:
            return (clean_url
                    .replace('github.com', 'raw.githubusercontent.com')
                    .replace('/tree/', '/refs/heads/')
                    + '/README.md')
        return (clean_url
                .replace('github.com', 'raw.githubusercontent.com')
                + f'/refs/heads/{branch}/README.md')

    # 首先尝试 main 分支
    try:
        main_url = build_readme_url('main')
        logger.info(f"Building README URL for main branch: {main_url}")
        logger.info(f"Fetching README from {main_url}")
        response = requests.get(main_url)
        if response.ok:
            return response.text
    except Exception as error:
        logger.error(f"Failed to fetch README from main branch: {error}")

    # 如果 main 分支失败，尝试 master 分支
    try:
        master_url = build_readme_url('master')
        logger.info(f"Fetching README from {master_url}")
        response = requests.get(master_url)
        if response.ok:
            return response.text
        raise Exception(f"Failed to fetch README: {response.status_code}")
    except Exception as error:
        raise Exception(f"Failed to fetch README from both main and master branches: {error}")



@mcp.tool()
def get_mcp_servers(server_created_date: str) -> List[dict]:
    """
    Query the MCP servers database and return servers created on the specified date.
    
    Args:
        server_created_date (str): The date to filter servers by (format: 'YYYY-MM-DD')
        
    Returns:
        List[dict]: A list of dictionaries containing server information
        
    Example:
        >>> servers = get_mcp_servers("2024-01-15")
        >>> print(servers)
        [
            {
                "id": 1,
                "name": "Server1",
                "description": "Description1",
                "created_at": "2024-01-15T10:00:00",
                "features": ["feature1", "feature2"],
                "github_url": "https://github.com/user/repo"
            },
            ...
        ]
    """
    try:
        # Use existing database session
        from .database import get_session
        from .types import Server

        session = get_session()

        # Query using SQLAlchemy ORM
        servers = (
            session.query(Server)
            .filter(text("DATE(created_at) = DATE(:created_date)"))
            .params(created_date=server_created_date)
            .order_by(Server.created_at.desc())
            .limit(10)
            .all()
        )

        # Convert to list of dictionaries using existing to_dict method
        result = []
        for server in servers:
            server_dict = server.to_dict()
            server_dict['README'] = fetch_readme_content(github_url=server.github_url)
            server_dict['internal_link'] = "https://mcpserver.cloud/server/" + server.slug
            result.append(server_dict)

        return result

    except SQLAlchemyError as e:
        return {"error": f"Database error: {str(e)}"}
    except Exception as e:
        return {"error": f"Unexpected error: {str(e)}"}


def run_server():
    if not os.getenv('POSTGRES_URL'):
        error_msg = "Error: POSTGRES_URL environment variable is not set"
        logger.error(error_msg)
        raise SystemExit(error_msg)

    mcp.run(transport='stdio')